@@ -62,8 +62,6 @@ typedef struct _WnckTaskClass   WnckTaskClass;
 #define DEFAULT_WIDTH 1
 #define DEFAULT_HEIGHT 48
 
-#define TIMEOUT_ACTIVATE 1000
-
 #define N_SCREEN_CONNECTIONS 5
 
 #define POINT_IN_RECT(xcoord, ycoord, rect) \
@@ -261,14 +259,21 @@ static void     wnck_tasklist_check_end_sequence       (WnckTasklist   *tasklist
 static gpointer task_parent_class;
 static gpointer tasklist_parent_class;
 
+/*
+ * Keep track of all tasklist instances so we can decide
+ * whether to show windows from all monitors in the
+ * tasklist
+ */
+static GSList *tasklist_instances;
+
 /**
- * eel_gtk_label_make_bold.
+ * make_gtk_label_bold.
  *
  * Switches the font of label to a bold equivalent.
  * @label: The label.
  **/
-void
-eel_gtk_label_make_bold (GtkLabel *label)
+static void
+make_gtk_label_bold (GtkLabel *label)
 {
   PangoFontDescription *font_desc;
 
@@ -287,8 +292,8 @@ eel_gtk_label_make_bold (GtkLabel *label)
   pango_font_description_free (font_desc);
 }
 
-void
-wnck_gtk_label_make_normal (GtkLabel *label)
+static void
+make_gtk_label_normal (GtkLabel *label)
 {
   PangoFontDescription *font_desc;
 
@@ -517,6 +522,11 @@ wnck_tasklist_init (WnckTasklist *tasklist)
   atk_obj = gtk_widget_get_accessible (widget);
   atk_object_set_name (atk_obj, _("Window List"));
   atk_object_set_description (atk_obj, _("Tool to switch between visible windows"));
+
+  tasklist_instances = g_slist_append (tasklist_instances, tasklist);
+  g_slist_foreach (tasklist_instances,
+		   (GFunc) wnck_tasklist_update_lists,
+		   NULL);
 }
 
 static void
@@ -545,6 +555,11 @@ wnck_tasklist_finalize (GObject *object)
   WnckTasklist *tasklist;
 
   tasklist = WNCK_TASKLIST (object);
+  
+  tasklist_instances = g_slist_remove (tasklist_instances, tasklist);
+  g_slist_foreach (tasklist_instances,
+		   (GFunc) wnck_tasklist_update_lists,
+		   NULL);
 
   if (tasklist->priv->free_icon_loader_data != NULL)
     (* tasklist->priv->free_icon_loader_data) (tasklist->priv->icon_loader_data);
@@ -1139,8 +1154,8 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
   while (l != NULL)
     {
       WnckTask *task = WNCK_TASK (l->data);
-      int col = i % n_cols;
-      int row = i / n_cols;
+      int row = i % n_rows;
+      int col = i / n_rows;
       
       child_allocation.x = total_width*col / n_cols;
       child_allocation.y = allocation->height*row / n_rows;
@@ -1181,6 +1196,8 @@ wnck_tasklist_realize (GtkWidget *widget)
 #endif
   
   (* GTK_WIDGET_CLASS (tasklist_parent_class)->realize) (widget);
+
+  wnck_tasklist_update_lists (tasklist);
 }
 
 static void
@@ -1498,26 +1515,36 @@ wnck_tasklist_update_lists (WnckTasklist *tasklist)
   GList *l;
   WnckTask *win_task;
   WnckTask *class_group_task;
-  gint monitor_num;
 
   wnck_tasklist_free_tasks (tasklist);
   
-  windows = wnck_screen_get_windows (tasklist->priv->screen);
-  
   if (GTK_WIDGET (tasklist)->window != NULL)
     {
-      monitor_num = gdk_screen_get_monitor_at_window (_wnck_screen_get_gdk_screen (tasklist->priv->screen),
-                                                      GTK_WIDGET (tasklist)->window);
-      if (monitor_num != tasklist->priv->monitor_num)
-        {
-          tasklist->priv->monitor_num = monitor_num;
-          gdk_screen_get_monitor_geometry (_wnck_screen_get_gdk_screen (tasklist->priv->screen),
-                                           tasklist->priv->monitor_num,
-                                           &tasklist->priv->monitor_geometry);
+      /*
+       * only show windows from this monitor if there is more than one tasklist running
+       */
+      if (tasklist_instances == NULL || tasklist_instances->next == NULL)
+	{
+	  tasklist->priv->monitor_num = -1;
         }
+      else
+	{
+	  int monitor_num;
+
+	  monitor_num = gdk_screen_get_monitor_at_window (_wnck_screen_get_gdk_screen (tasklist->priv->screen),
+							  GTK_WIDGET (tasklist)->window);
+
+	  if (monitor_num != tasklist->priv->monitor_num)
+	    {
+	      tasklist->priv->monitor_num = monitor_num;
+	      gdk_screen_get_monitor_geometry (_wnck_screen_get_gdk_screen (tasklist->priv->screen),
+					       tasklist->priv->monitor_num,
+					       &tasklist->priv->monitor_geometry);
+	    }
+	}
     }
  
-  l = windows;
+  l = windows = wnck_screen_get_windows (tasklist->priv->screen);
   while (l != NULL)
     {
       win = WNCK_WINDOW (l->data);
@@ -1761,7 +1788,8 @@ wnck_tasklist_window_changed_geometry (WnckWindow   *window,
    * the tasklist itself possibly changed monitor.
    */
   monitor_changed = FALSE;
-  if (wnck_window_get_state (window) & WNCK_WINDOW_STATE_SKIP_TASKLIST &&
+  if (tasklist->priv->monitor_num != -1 &&
+      (wnck_window_get_state (window) & WNCK_WINDOW_STATE_SKIP_TASKLIST) &&
       GTK_WIDGET (tasklist)->window != NULL)
     {
       /* Do the extra check only if there is a suspect of a monitor change (= this window is off monitor) */
@@ -1885,7 +1913,7 @@ wnck_task_menu_activated (GtkMenuItem *menu_item,
 }
 
 static void
-wnck_tasklist_activate_task_window     (WnckTask *task)
+wnck_tasklist_activate_task_window (WnckTask *task)
 {
   WnckTasklist *tasklist;
   WnckWindowState state;
@@ -1913,7 +1941,7 @@ wnck_tasklist_activate_task_window     (WnckTask *task)
     }
   else
     {
-      if (task->was_active)
+      if (task->was_active || wnck_window_transient_is_active (task->window))
 	{
 	  task->was_active = FALSE;
 	  wnck_window_minimize (task->window);
@@ -1941,6 +1969,53 @@ wnck_tasklist_activate_task_window     (WnckTask *task)
   wnck_tasklist_change_active_task (tasklist, task);
 }
 
+static void 
+wnck_task_close_all (GtkMenuItem *menu_item,
+ 		     gpointer     data)
+{
+  WnckTask *task = WNCK_TASK (data);
+  GList *l;
+
+  l = task->windows;
+  while (l)
+    {
+      WnckTask *child = WNCK_TASK (l->data);
+      wnck_window_close (child->window, gtk_get_current_event_time ());
+      l = l->next;
+    }
+}
+
+static void
+wnck_task_unminimize_all (GtkMenuItem *menu_item,
+		          gpointer     data)
+{
+  WnckTask *task = WNCK_TASK (data);
+  GList *l;
+
+  l = task->windows;
+  while (l)
+    {
+      WnckTask *child = WNCK_TASK (l->data);
+      wnck_window_unminimize (child->window);
+      l = l->next;
+    }
+}
+
+static void 
+wnck_task_minimize_all (GtkMenuItem *menu_item,
+  		        gpointer     data)
+{
+  WnckTask *task = WNCK_TASK (data);
+  GList *l;
+
+  l = task->windows;
+  while (l)
+    {
+      WnckTask *child = WNCK_TASK (l->data);
+      wnck_window_minimize (child->window);
+      l = l->next;
+    }
+}
 
 static void
 wnck_task_popup_menu (WnckTask *task,
@@ -1994,20 +2069,64 @@ wnck_task_popup_menu (WnckTask *task,
       
       gtk_widget_show (menu_item);
 
-       if (action_submenu)
-         gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item),
-                                    wnck_create_window_action_menu (win_task->window));
-       else
-         g_signal_connect_object (G_OBJECT (menu_item), "activate",
-                                  G_CALLBACK (wnck_task_menu_activated),
-                                  G_OBJECT (win_task),
-                                  0);
+      if (action_submenu)
+        gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item),
+                                   wnck_create_window_action_menu (win_task->window));
+      else
+        g_signal_connect_object (G_OBJECT (menu_item), "activate",
+                                 G_CALLBACK (wnck_task_menu_activated),
+                                 G_OBJECT (win_task),
+                                 0);
       
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
       
       l = l->next;
     }
 
+  /* In case of Right click, show Close All, Minimize All, Unminimize All*/
+  if (action_submenu) 
+    {
+      GtkWidget *separator;
+      GtkWidget *image;
+		
+      separator = gtk_separator_menu_item_new ();
+      gtk_widget_show (separator);
+      gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), separator);
+
+      menu_item = gtk_image_menu_item_new_with_mnemonic(_("_Close All"));
+      image = gtk_image_new_from_stock (WNCK_STOCK_DELETE, GTK_ICON_SIZE_MENU);
+      gtk_widget_show (image);
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), image);    
+      gtk_widget_show (menu_item);
+      gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+      g_signal_connect_object (G_OBJECT (menu_item), "activate",
+			       G_CALLBACK (wnck_task_close_all),
+			       G_OBJECT (task),
+			       0);
+
+      separator = gtk_separator_menu_item_new ();
+      gtk_widget_show (separator);
+      gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), separator);
+	    
+      menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Minimize All"));
+      image = gtk_image_new_from_stock (WNCK_STOCK_MINIMIZE, GTK_ICON_SIZE_MENU);
+      gtk_widget_show (image);
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), image);  	
+      gtk_widget_show (menu_item);
+      gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+      g_signal_connect_object (G_OBJECT (menu_item), "activate",
+	    		       G_CALLBACK (wnck_task_minimize_all),
+			       G_OBJECT (task),
+			       0);
+		
+      menu_item =  gtk_image_menu_item_new_with_mnemonic (_("_Unminimize All"));
+      gtk_widget_show (menu_item);
+      gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);			
+      g_signal_connect_object (G_OBJECT (menu_item), "activate",
+  			       G_CALLBACK (wnck_task_unminimize_all),
+			       G_OBJECT (task),
+			       0);							
+    }
   gtk_menu_set_screen (GTK_MENU (menu),
 		       _wnck_screen_get_gdk_screen (task->tasklist->priv->screen));
   
@@ -2290,9 +2409,9 @@ wnck_task_update_visible_state (WnckTask *task)
     {
       gtk_label_set_text (GTK_LABEL (task->label), text);
       if (wnck_task_get_demands_attention (task))
-        eel_gtk_label_make_bold ((GTK_LABEL (task->label)));
+        make_gtk_label_bold ((GTK_LABEL (task->label)));
       else
-        wnck_gtk_label_make_normal ((GTK_LABEL (task->label)));
+        make_gtk_label_normal ((GTK_LABEL (task->label)));
       gtk_tooltips_set_tip (task->tasklist->priv->tooltips, task->button, text, NULL);
       g_free (text);
     }
@@ -2413,7 +2532,7 @@ wnck_task_drag_motion (GtkWidget          *widget,
 {
 
    if (task->button_activate == 0 && task->type == WNCK_TASK_WINDOW)
-       task->button_activate = g_timeout_add (TIMEOUT_ACTIVATE,
+       task->button_activate = g_timeout_add (WNCK_ACTIVATE_TIMEOUT,
                                               wnck_task_motion_timeout,
                                               task);
    gdk_drag_status (context,0,time);
@@ -2528,8 +2647,12 @@ wnck_task_create_widgets (WnckTask *task)
 
   text = wnck_task_get_text (task);
   task->label = gtk_label_new (text);
+  gtk_misc_set_alignment (GTK_MISC (task->label), 0.0, 0.5);
+  gtk_label_set_ellipsize (GTK_LABEL (task->label),
+                          PANGO_ELLIPSIZE_END);
+
   if (wnck_task_get_demands_attention (task))
-    eel_gtk_label_make_bold ((GTK_LABEL (task->label)));
+    make_gtk_label_bold ((GTK_LABEL (task->label)));
   gtk_widget_show (task->label);
 
   gtk_table_attach (GTK_TABLE (table),
@@ -2537,13 +2660,13 @@ wnck_task_create_widgets (WnckTask *task)
 		    0, 1,
 		    0, 1,
 		    0, GTK_EXPAND,
-		    0, 0);
+		    4, 0);
   gtk_table_attach (GTK_TABLE (table),
 		    task->label,
 		    1, 2,
 		    0, 1,
-		    GTK_FILL, GTK_EXPAND,
-		    4, 0);
+		    GTK_FILL | GTK_EXPAND, GTK_EXPAND,		    
+		    2, 0);
 
   gtk_container_add (GTK_CONTAINER (task->button), table);
   gtk_widget_show (table);
@@ -2608,7 +2731,7 @@ draw_dot (GdkWindow *window, GdkGC *lgc, GdkGC *dgc, int x, int y)
 }
 
 
-gboolean
+static gboolean
 wnck_task_class_group_expose (GtkWidget        *widget,
 			      GdkEventExpose   *event,
 			      gpointer          data)
@@ -2651,11 +2774,10 @@ wnck_task_compare (gconstpointer  a,
 {
   WnckTask *task1 = WNCK_TASK (a);
   WnckTask *task2 = WNCK_TASK (b);
-  gulong xid1_1, xid1_2;
-  gulong xid2_1, xid2_2;
-  
-  xid1_1 = xid1_2 = xid2_1 = xid2_2 = 0;   /* silence compiler */
-  
+  gint pos1, pos2;
+
+  pos1 = pos2 = 0;  /* silence the compiler */
+
   switch (task1->type)
     {
     case WNCK_TASK_CLASS_GROUP:
@@ -2665,11 +2787,10 @@ wnck_task_compare (gconstpointer  a,
 	return -1; /* Sort groups before everything else */
 
     case WNCK_TASK_WINDOW:
-      xid1_1 = wnck_window_get_group_leader (task1->window);
-      xid1_2 = wnck_window_get_xid (task1->window);
+      pos1 = wnck_window_get_sort_order (task1->window);
       break;
     case WNCK_TASK_STARTUP_SEQUENCE:
-      xid1_1 = xid1_2 = G_MAXULONG;
+      pos1 = G_MAXINT;
       break;
     }
 
@@ -2682,20 +2803,20 @@ wnck_task_compare (gconstpointer  a,
 	return 1; /* Sort groups before everything else */
 
     case WNCK_TASK_WINDOW:
-      xid2_1 = wnck_window_get_group_leader (task2->window);
-      xid2_2 = wnck_window_get_xid (task2->window);
+      pos2 = wnck_window_get_sort_order (task2->window);
       break;
     case WNCK_TASK_STARTUP_SEQUENCE:
-      xid2_1 = xid2_2 = G_MAXULONG;
+      pos2 = G_MAXINT;
       break;
     }
       
-  if ((xid1_1 < xid2_1) || ((xid1_1 == xid2_1) && (xid1_2 < xid2_2)))
+  if (pos1 < pos2)
     return -1;
-  else if ((xid1_1 == xid2_1) && (xid1_2 == xid2_2))
-    return 0;
-  else
+  else if (pos1 > pos2)
     return 1;
+  else
+    return 0; /* should only happen if there's multiple processes being
+               * started, and then who cares about sort order... */
 }
 
 static void
