@@ -12,12 +12,13 @@ main (int argc, char **argv)
   
   gtk_init (&argc, &argv);
 
-  screen = wnck_screen_get (0);
+  screen = wnck_screen_get_default ();
   
   /* because the pager doesn't respond to signals at the moment */
   wnck_screen_force_update (screen);
   
   win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_default_size (GTK_WINDOW (win), 200, 100);
   gtk_window_stick (GTK_WINDOW (win));
   /*   wnck_gtk_window_set_dock_type (GTK_WINDOW (win)); */
 
@@ -32,7 +33,7 @@ main (int argc, char **argv)
 
   tasklist = wnck_tasklist_new (screen);
 
-  wnck_tasklist_set_grouping (WNCK_TASKLIST (tasklist), WNCK_TASKLIST_AUTO_GROUP);
+  wnck_tasklist_set_grouping (WNCK_TASKLIST (tasklist), WNCK_TASKLIST_ALWAYS_GROUP);
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
   gtk_container_add (GTK_CONTAINER (win), frame);
