@@ -144,6 +144,7 @@ WnckClassGroup *wnck_window_get_class_group (WnckWindow *window);
 const char* wnck_window_get_session_id        (WnckWindow *window);
 const char* wnck_window_get_session_id_utf8   (WnckWindow *window);
 int         wnck_window_get_pid               (WnckWindow *window);
+gint        wnck_window_get_sort_order        (WnckWindow *window);
 
 WnckWindowType wnck_window_get_window_type    (WnckWindow *window);
 
@@ -195,7 +196,7 @@ void     wnck_window_activate  (WnckWindow *window);
 gboolean wnck_window_is_active (WnckWindow *window);
 gboolean wnck_window_is_most_recently_activated (WnckWindow *window);
 void     wnck_window_activate_transient (WnckWindow *window);
-
+gboolean wnck_window_transient_is_active (WnckWindow *window);
 
 GdkPixbuf* wnck_window_get_icon      (WnckWindow *window);
 GdkPixbuf* wnck_window_get_mini_icon (WnckWindow *window);
